from sum_pts.pt_counter import *

folder = pathlib.Path('.') / 'ex'

with open(folder / 'case1/kwargs_case1.json', 'w') as f:
    json.dump(dict(prefix='# custom-prefix',
                   left='left',
                   right='right',
                   points='(penguins|pts)'), indent=4, fp=f)

with open(folder / 'case2/kwargs_case2.json', 'w') as f:
    json.dump(dict(prefix=' *\\\\prob',
                   left='\[',
                   right='\]',
                   points='pts',
                   rm_list=['\\(\d+.?\d* each\\)',
                            '\\((\d+.?\d*,? ?)+\\)',
                            '\\{', '\\}', ':']), indent=4, fp=f)

with open(folder / 'case3/kwargs_case3.json', 'w') as f:
    json.dump(dict(ignore_case=True,
                   points='(pts?|points?)'), indent=4, fp=f)

with open(folder / 'case4/kwargs_case4.json', 'w') as f:
    json.dump(dict(left=' \\('), indent=4, fp=f)


def load_clean_csv(f_csv):
    df = pd.read_csv(f_csv, index_col=0)
    for col in df.columns:
        if 'Unnamed' in col:
            df.rename({col: ''}, axis=1, inplace=True)
    df.fillna('', inplace=True)

    return df


def test_point_counter():
    for case_folder in folder.glob('case*'):
        # get first py and ipynb file in each case folder (its unique)
        file_list = list()
        for pattern in ('*.py', '*.ipynb', '*.tex'):
            file_list += case_folder.glob(pattern)

        # load kwargs to parse (if available)
        f_kwargs = next(case_folder.glob('kwargs_case*.json'), False)
        if f_kwargs:
            with open(f_kwargs) as f:
                kwargs = json.load(f)
        else:
            kwargs = dict()

        for file in sorted(file_list):
            # observed
            pc = PointCounter()
            pc.parse_file(file, **kwargs)
            df_obs = pc.to_df()

            # expected
            df_exp = load_clean_csv(case_folder / 'expected.csv')

            try:
                pd.testing.assert_frame_equal(df_obs, df_exp,
                                              check_dtype=False)
            except:
                print('hi')



import json
import pathlib
import re
from copy import deepcopy

import pandas as pd


class PointCounter:
    """ counts point in assignment. """

    def __init__(self):
        self.df = pd.DataFrame()
        self.df.index.name = 'part'

    def parse_file(self, file, **kwargs):
        file = pathlib.Path(file)
        with open(file) as f:
            if file.suffix == '.ipynb':
                # read in json file
                json_dict = json.load(f)

                # parse each cell
                for cell in json_dict['cells']:
                    self.parse(s=''.join(cell['source']), **kwargs)
            else:
                # parse contents of file
                self.parse(f.read(), **kwargs)

    def parse(self, s, pt_split=None, left=None, right=None, prefix=None,
              points=None, ignore_case=None, rm_list=None):
        """ parses text to find points

        # part 1 (3 pt + 10 extra credit)

        Args:
            s (str): text to be parsed
            pt_split (str): python regex representing all separations
                between point types (e.g. normal vs extra credit)
            left (str): python regex matches left of a point block
            right (str): python regex matches right of point block
            prefix (str): python regex matches start of line containing points
            points (str): python regex matches once somewhere within point
                block
            ignore_case (bool): ignores case of inputs if passed
            rm_list (list): list of regex strings to remove from
        """
        # defaults stored here to avoid redundancy in __main__
        if pt_split is None:
            pt_split = '[+&,]'
        if left is None:
            left = '\('
        if right is None:
            right = '\)'
        if prefix is None:
            prefix = '#'
        if points is None:
            points = '([Pp]ts?|[Pp]oints?)'
        if rm_list is None:
            rm_list = list()
        if ignore_case is None:
            ignore_case = False

        flag_dict = {'flags': re.IGNORECASE} if ignore_case else dict()

        # a pts_block contains all point value (and type) information.
        # e.g. "(3 points + 1 extra credit)" has 3 '' points and 1 'extra
        # credit' points
        pts_block = f'{left}.*{points}.*{right}'

        for line in re.finditer(f'{prefix}.*{pts_block}.*', s, **flag_dict):
            line = line.group()

            # extract name of problem
            prob_name = line
            for s in [pts_block, prefix] + rm_list:
                prob_name = re.sub(s, '', prob_name, **flag_dict).strip()

            # extract & clean point values (all point types)
            str_pts = re.search(pts_block, line, **flag_dict).group()
            for s in ['^' + left, right + '$', points] + rm_list:
                str_pts = re.sub(s, '', str_pts, **flag_dict)

            # record point values (per point type)
            for s in re.split(pt_split, str_pts, **flag_dict):
                # extract pts
                item_pts = re.search('\d+\.?\d*', s, **flag_dict).group()

                # extract name of points
                item_name = s.replace(item_pts, '').strip()

                # record
                self._record(prob_name=prob_name, item_name=item_name,
                             item_pts=item_pts)

    def _record(self, prob_name, item_name, item_pts, check_doesnt_exist=True):
        if check_doesnt_exist:
            if prob_name in self.df.index and item_name in self.df.columns:
                # entry exists, validate it isn't populated
                assert pd.isna(self.df.loc[prob_name, item_name]), \
                    f'duplicate entry for {prob_name} {item_name}'

            # record item_pts
            self.df.loc[prob_name, item_name] = float(item_pts)

    def to_df(self):
        df = deepcopy(self.df)

        # add total final col (if there are multiple cols)
        if df.shape[1] > 1:
            df['part total'] = df.sum(axis=1)

        # add total final row
        df.loc['total', :] = df.sum(axis=0)

        df.fillna('', inplace=True)

        return df



import json
import pathlib
import shlex
import tempfile

import pandas as pd

import sum_pts
from sum_pts.__main__ import main, parser
from test_pt_counter import load_clean_csv


def test_main():
    # we run case2 via the command line

    folder_case = pathlib.Path(sum_pts.__file__).parents[1] / 'test/ex/case2'
    file_in = folder_case / 'ex_hw2.tex'

    with open(folder_case / 'kwargs_case2.json') as f:
        kwargs = json.load(f)

    # build command
    file_csv = tempfile.NamedTemporaryFile(suffix='.csv').name
    file_md = tempfile.NamedTemporaryFile(suffix='.md').name
    cmd = f'{file_in} -c {file_csv} -m {file_md} -q'
    cmd += " --left '{left}' --right '{right}'".format(**kwargs)
    cmd += " --points '{points}' --prefix '{prefix}'".format(**kwargs)
    for s in kwargs['rm_list']:
        cmd += f" -r '{s}'"

    # run argparser to ensure cmd is formed properly
    result = parser.parse_args(shlex.split(cmd))
    # ensure things are parsed properly
    for key, val in kwargs.items():
        assert getattr(result, key) == val, f'parser error: {key}'

    # run our own parser & check output
    main(shlex.split(cmd))
    df_exp = load_clean_csv(folder_case / 'expected.csv')
    df_obs = load_clean_csv(file_csv)
    pd.testing.assert_frame_equal(df_obs, df_exp, check_dtype=False)



import argparse

import sum_pts

description = ('counts assignment points, '
               'see https://github.com/matthigger/sum_pts for further details')
parser = argparse.ArgumentParser(prog='sum_pts',
                                 description=description)
parser.add_argument('file_in', type=str, help='input file')
parser.add_argument('-c', '--csv-out', type=str, help='output csv file',
                    dest='file_csv', default=None)
parser.add_argument('-m', '--md-out', type=str, help='output markdown file',
                    dest='file_md', default=None)
parser.add_argument('-q', '--quiet', action='store_true', dest='quiet',
                    help='disables printing markdown output to command line')
parser.add_argument('-aA', '--ignore_case', action='store_true',
                    dest='ignore_case', help='case ignored in all regex '
                                             'comparisons', default=None)
parser.add_argument('--left', type=str, default=None, dest='left',
                    help='regex matches left of a point block')
parser.add_argument('--right', type=str, default=None, dest='right',
                    help='regex matches right of a point block')
parser.add_argument('--prefix', type=str, default=None, dest='prefix',
                    help='regex matches lines containing points')
parser.add_argument('--points', type=str, default=None, dest='points',
                    help='regex matches "points" or similar')
parser.add_argument('-r', '--rm', action='append', dest='rm_list',
                    help='regex to be discarded in points line')


def main(args=None):
    args = parser.parse_args(args)

    # parse file, get dataframe
    pc = sum_pts.PointCounter()
    pc.parse_file(file=args.file_in,
                  ignore_case=args.ignore_case,
                  left=args.left,
                  right=args.right,
                  prefix=args.prefix,
                  points=args.points,
                  rm_list=args.rm_list)
    df = pc.to_df()

    if args.file_csv is not None:
        # write csv to file
        df.to_csv(args.file_csv)

    # get markdown table
    s_markdown = df.to_markdown()

    if not args.quiet:
        print(s_markdown)

    if args.file_md is not None:
        # write markdown to file
        with open(args.file_md, 'w') as f:
            print(s_markdown, file=f)


if __name__ == '__main__':
    main()
